# !/usr/bin/env python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-23
"""
Functions for getting user input.
"""
import six

import benchline.args


def validate_args(parser, options, args):
    pass


def select(prompt, valid_values):
    """
    Ask the user to select one of valid_values

    :param prompt: string presented to the user
    :param valid_values: list of strings
    :return: one of the string in valid_values
    """
    response = six.moves.input("%s [%s]: " % (prompt, ", ".join(valid_values)))
    if response in valid_values:
        return response
    else:
        return select(prompt, valid_values)


def get_int(prompt):
    """Prompts the user for an int until they give one.
    :param prompt: string
    :return: the entered int
    """
    ans = six.moves.input(prompt)
    try:
        return int(ans)
    except ValueError:
        return get_int(prompt)


def main():
    benchline.args.go(__doc__, validate_args=validate_args)


if __name__ == "__main__":
    main()



# !/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-20
"""
Methods to interact with directories.
"""
import os
import random

import benchline.args


def dir_file_in(file_name):
    """Returns the OS directory where file_name resides.
    To return the directory of the current file use __file__ as
    file_name.

    >>> filename = str(random.random())
    >>> cwd = os.getcwd()
    >>> open(filename, "w").close()
    >>> dir_file_in(filename) == cwd
    True
    >>> os.remove(filename)
    """
    return os.path.dirname(os.path.realpath(file_name))


if __name__ == "__main__":
    benchline.args.go(__doc__)



# !/usr/bin/env python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-23
"""
Installs the changes to a python project to the local machine.

Just a convenience script.
"""
import benchline.args
import benchline.command


def validate_args(parser, options, args):
    pass


def main():
    options, args = benchline.args.go(__doc__, validate_args=validate_args)
    if options.doctest:
        benchline.command.run("python setup.py test")
        benchline.command.run("python3 setup.py test")
    benchline.command.run("sudo python3 setup.py install")
    benchline.command.run("sudo python setup.py install")


if __name__ == "__main__":
    main()



# !/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: Mon Aug 4, 2008
"""
Script to create new python files quickly.
"""

import os
import sys
import datetime
from string import Template

import six

import benchline.files
import benchline.args
import benchline.command


TEMPLATE_DIR = benchline.files.dir_file_in(__file__)


class NewPythonFile(object):
    """
    This class automates the creation of other new python files
    according to a pre-defined template.

    It also allows for the creation of skeleton test cases
    so that creating unit tests is less time consuming.
    """

    def _make_class_name_from_file_name(self, file_name):
        class_name = os.path.basename(file_name.replace(".py", "").replace("_", " ").title().replace(" ", ""))
        return class_name

    def _get_template_path(self, template_type):
        if template_type == "class":
            template_filename = 'new_class_file.tmpl'
        elif template_type == "cmd":
            template_filename = 'new_cmd_file.tmpl'
        else:
            template_filename = 'new_python_file.tmpl'

        return os.path.join(TEMPLATE_DIR, template_filename)

    def _ask_to_overwrite(self, new_file_name):
        if os.path.exists(new_file_name):
            ans = raw_input("Overwrite %s? " % new_file_name)
            if ans.lower() != "y":
                six.print_("Not overwriting.  %s untouched" % new_file_name)
                sys.exit(0)

    def _write_new_file(self, new_file_name, template):
        class_name = self._make_class_name_from_file_name(new_file_name)
        t = Template(open(self._get_template_path(template), "r").read())
        d = dict(class_name=class_name, date=datetime.date.today().strftime("%Y-%m-%d"))
        fo = open(new_file_name, "w")
        fo.write(t.substitute(d))
        fo.close()

    def make_new_python_file(self, new_file_name, template, noedit, overwrite):
        new_file_name = os.path.realpath(new_file_name)
        if not overwrite:
            self._ask_to_overwrite(new_file_name)
        self._write_new_file(new_file_name, template)
        six.print_("Wrote %s using %s as a template" % (new_file_name, template))
        if not noedit:
            benchline.command.run("vim %s" % new_file_name)


def validate_args(parser, option, args):
    if len(args) < 1:
        parser.error("Please specify the filename of the file to create.")


def main():
    parser = benchline.args.make_parser(__doc__)
    parser.add_option("--template",
                              help="template file to use (one of 'functional'(default), 'class', 'cmd')",
                              default="functional")
    parser.add_option("-n", "--noedit", help="do not edit after create [False]", default=False,
                              action="store_true")
    parser.add_option("-o", "--overwrite", help="overwrite existing files without prompting [False]",
                              default=False,
                              action="store_true")
    options, args = benchline.args.triage(parser, validate_args=validate_args)
    n = NewPythonFile()
    n.make_new_python_file(args[0], options.template,
                           options.noedit, options.overwrite)


if __name__ == "__main__":
    main()



# !/usr/bin/env python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-25
"""
Script to create a new setup.py file.
"""
from string import Template
import os

import six

import benchline.args
import benchline.user_input


def validate_args(parser, options, args):
    if len(args) < 1:
        parser.error("The first positional argument must be the project name.")


def get_setup_py(project_str, version_str="0.0.1"):
    template = Template("""# !/usr/bin/env python

from setuptools import setup, find_packages

# we only use the subset of markdown that is also valid reStructuredText so
# that our README.md works on both github (markdown) and pypi (reStructuredText)
with open("README.md") as rm_file:
    long_description = rm_file.read()

setup(name='${project}',
      version='${version}',
      description="TODO",
      long_description=long_description,
      author='TODO',
      author_email='TODO',
      url='TODO',
      packages=find_packages(),
      data_files=[('', ['README.md', 'LICENSE'],)],
      test_suite="${project}.test",
      license="MIT",
      install_requires=['TODO'],
      zip_safe=False,
      entry_points={
          'console_scripts': [
              'TODO = ${project}.todo:main']}
)
""")
    return template.substitute(project=project_str, version=version_str)


def write_setup_py(contents):
    """
    :param contents: setup.py contents
    :return: void
    """
    open("setup.py", "w").write(contents)


def main():
    parser = benchline.args.make_parser(usage="usage: %%prog [options] project_name\n%s" % __doc__)
    parser.add_option("--version", action="store_true",
                      help="version to put in the file. default=0.0.1", default="0.0.1")
    options, args = benchline.args.triage(parser, validate_args=validate_args)
    setup_py_str = get_setup_py(args[0], options.version)
    if os.path.exists("setup.py"):
        yn = benchline.user_input.select("setup.py exists.  overwrite?", ('y', 'n'))
        if yn == 'y':
            write_setup_py(setup_py_str)
        else:
            six.print_("no changes made to setup.py.  existing...")
    else:
        write_setup_py(setup_py_str)


if __name__ == "__main__":
    main()



# !/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-20
"""
Methods to call os commands more easily.
"""

import subprocess

import benchline.args


def run(cmd, successful_ret_codes=(0,), **kwargs):
    """Wrapper to subprocess.call mimicking subprocess.check_call allowing us to
    specify other successful return codes other than just 0.

    >>> run("ls")
    0

    :param kwargs:
    :param p_open_args:
    :param successful_ret_codes:
    :type successful_ret_codes: list of int
    :return: 0 on success and CalledProcessError Exception on error
    """
    ret_code = subprocess.call(cmd, shell=True, **kwargs)
    if ret_code and ret_code not in successful_ret_codes:
        raise subprocess.CalledProcessError(ret_code, cmd)
    return 0


def output(cmd, successful_ret_codes=(0,), **kwargs):
    """
    Returns the output from cmd as a string

    >>> subprocess.check_output("ls", shell=True) == output("ls")
    True

    :param cmd: an os command to run
    :return: the String output from cmd
    """
    if 'stdout' in kwargs:
        raise ValueError('stdout argument not allowed, it will be overridden.')
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                               shell=True, **kwargs)
    output, unused_err = process.communicate()
    ret_code = process.poll()
    if ret_code and ret_code not in successful_ret_codes:
        raise subprocess.CalledProcessError(ret_code, cmd,
                                            output=output)
    return output



if __name__ == "__main__":
    benchline.args.go(__doc__)



# !/usr/bin/env python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-22
"""
Command-line script and library to automate the steps involved to
deploy python packages automatically to pypi and github.
"""

import six

import benchline.args
import benchline.command
import benchline.user_input

import os.path


def validate_args(parser, options, args):
    pass


def run_tests(version_str):
    """
    Run the tests through setuptools
    :param version_str:
    :return:
    """
    benchline.command.run("python%s setup.py test" % version_str)


def get_version():
    """
    Returns the version from the setup.py file.

    >>> six.b(".") in get_version()
    True

    :return: string version of setup.py file
    """
    return benchline.command.output("python setup.py --version")[:-1]


def increment_revision():
    """
    Opens the setup.py file in vim for editing.
    :return:
    """
    benchline.command.run("vim setup.py")


def get_pypi_servers():
    """
    Returns all the pypi servers in the .pypirc file.
    :return: string
    """
    config = six.moves.configparser.SafeConfigParser()
    config.read(os.path.join(os.path.expanduser("~"), ".pypirc"))
    return config.get("distutils", "index-servers")[1:].split("\n")


def get_pypi_server():
    servers = get_pypi_servers()
    return benchline.user_input.select("Select the pypi server to deploy to", servers)


def main():
    benchline.args.go(__doc__, validate_args=validate_args)
    run_tests("2")
    run_tests("3")
    increment_revision()
    version = get_version()
    benchline.command.run("git commit -a -m \"[Updating version to %s]\"" % version)
    benchline.command.run("git tag %s" % version)
    benchline.command.run("git push")
    benchline.command.run("git push --tags")
    pypi_server = get_pypi_server()
    benchline.command.run("python setup.py sdist bdist_egg bdist_wheel upload -r %s" % pypi_server)
    benchline.command.run("python3 setup.py bdist_egg upload -r %s" % pypi_server)


if __name__ == "__main__":
    main()



# !/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-17
"""
Prints the sum of a timelog in hours.
"""
import datetime
import benchline.args
import six


def parse_times(line_str):
    """
    >>> parse_times("Mar 11: IN - 06:48 AM OUT - 05:28 PM")
    ('06:48 AM', '05:28 PM')
    """
    first_part, time_out_str = line_str.split(" OUT - ")
    time_in_str = first_part.split(" IN - ")[1]
    return time_in_str, time_out_str


def time_str_2_time(time_str):
    """
    >>> time_str_2_time("06:48 AM")
    datetime.datetime(1900, 1, 1, 6, 48)
    """
    return datetime.datetime.strptime(time_str, "%I:%M %p")


def time_delta_2_hours(time_delta):
    """
    >>> time_delta_2_hours(datetime.timedelta(0, 1800))  # 1800 seconds = 30 minutes
    0.5
    """
    return time_delta.seconds / float(3600)


def line2hours(line_str):
    """
    >>> line2hours("Mar 11: IN - 06:48 AM OUT - 05:28 PM")
    10.666666666666666
    """
    time_in, time_out = map(time_str_2_time, parse_times(line_str))
    delta = time_out - time_in
    return time_delta_2_hours(delta)


def validate_args(parser, options, args):
    if len(args) == 0:
        parser.error("The first argument must be the filename of the zylun timelog.")


def get_lines(file_name_str):
    """
    >>> get_lines("benchline/test/test_timelog.txt")
    ['Mar 11: IN - 06:48 AM OUT - 05:28 PM', 'Mar 12: IN - 06:56 AM OUT - 04:20 PM', 'Mar 13: IN - 06:58 AM OUT - 05:15 PM', 'Mar 14: IN - 06:58 AM OUT - 04:52 PM', 'Mar 15: IN - 06:56 AM OUT - 05:31 PM']
    """
    f = open(file_name_str)
    return [line[:-1] for line in f.readlines()]


def main():
    options, args = benchline.args.go(usage="usage: %%prog [options] timelog_file\n%s" % __doc__, validate_args=validate_args)
    lines = get_lines(args[0])
    hours = map(line2hours, lines)
    six.print_(reduce(lambda x, y: x + y, hours))


if __name__ == "__main__":
    main()



__author__ = 'paul'

import unittest
import doctest

import benchline.args
import benchline.command
import benchline.date_diff
import benchline.files
import benchline.new_python_file.new_python_file
import benchline.python_deploy
import benchline.python_install
import benchline.user_input
import benchline.hours_seconds_2_hours
import benchline.countdown_generator
import benchline.sum_timelog
import benchline.new_setup_py


def load_tests(loader, tests, ignore):
    tests.addTests(doctest.DocTestSuite(benchline.args))
    tests.addTests(doctest.DocTestSuite(benchline.command))
    tests.addTests(doctest.DocTestSuite(benchline.date_diff))
    tests.addTests(doctest.DocTestSuite(benchline.files))
    tests.addTests(doctest.DocTestSuite(benchline.new_python_file.new_python_file))
    tests.addTests(doctest.DocTestSuite(benchline.python_deploy))
    tests.addTests(doctest.DocTestSuite(benchline.python_install))
    tests.addTests(doctest.DocTestSuite(benchline.user_input))
    tests.addTests(doctest.DocTestSuite(benchline.hours_seconds_2_hours))
    tests.addTests(doctest.DocTestSuite(benchline.countdown_generator))
    tests.addTests(doctest.DocTestSuite(benchline.sum_timelog))
    tests.addTests(doctest.DocTestSuite(benchline.new_setup_py))
    return tests


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-24
"""
Script to create the letters for the alphabetic version of countdown,
the game show.
"""
import random
import string
import six

import benchline.args
import benchline.user_input


def validate_args(parser, options, args):
    pass


def random_from_list(lst):
    """Returns a random element in lst

    >>> lst = ['a', 'b', 'c']
    >>> random_from_list(lst) in lst
    True

    :param lst: list of strings
    :return: one random item in the list of strings
    """
    index = random.randint(0, len(lst) - 1)
    return lst[index]


def gen_random_letters(consonant_num, vowel_num):
    """
    Generate a random list of letters with consonant_num consonants and vowel_num vowels

    >>> len(gen_random_letters(1, 3)) == 4
    True

    :param consonant_num:
    :param vowel_num:
    :return: list of letters
    """
    alphabet = [let for let in string.ascii_lowercase]
    vowels = [let for let in "aeiouy"]
    consonants = list(set(alphabet).difference(vowels))
    return_list = []
    for let in range(consonant_num):
        return_list.append(random_from_list(consonants))
    for let in range(vowel_num):
        return_list.append(random_from_list(vowels))
    random.shuffle(return_list)
    return return_list


def main():
    benchline.args.go(__doc__, validate_args=validate_args)
    num_consonants = benchline.user_input.get_int("Enter the number of consonants: ")
    num_vowels = benchline.user_input.get_int("Enter the number of vowels: ")
    six.print_(" ".join(gen_random_letters(num_consonants, num_vowels)))


if __name__ == "__main__":
    main()



# !/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-17
"""
Converts hours and seconds like 10:11 to
hours (decimal).
"""
import six

import benchline.args


def mins2hours(minute_str):
    """
    >>> mins2hours("30")
    0.5
    """
    return float(minute_str) / 60.0


def hours_mins_2_hours(hours_minutes_str):
    """
    >>> hours_mins_2_hours("10:30")
    10.5
    """
    hour_str, minute_str = hours_minutes_str.split(":")
    return int(hour_str) + mins2hours(minute_str)


def validate_args(parser, options, args):
    if len(args) == 0:
        parser.error("The first argument must be hours and seconds like '10:11'")


def main():
    options, args = benchline.args.go(__doc__, validate_args=validate_args)
    six.print_(hours_mins_2_hours(args[0]))


if __name__ == "__main__":
    main()




#!/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-19
"""
Calculates the difference between two dates in days, months and years
and prints them.
"""
import datetime

import ago
import six

import benchline.args


def _valid_date_str(date_str):
    """
    >>> _valid_date_str("2014-02-02")
    True
    >>> _valid_date_str("watermelon")
    False

    :param date_str:
    :return: boolean
    """
    return len(date_str) == 10 and "-" in date_str


def validate_args(parser, options, args):
    if len(args) < 1:
        parser.error("The positional arguments must one or two date strings.")
    if not reduce(lambda x, y: x and y, map(_valid_date_str, args)):
        parser.error("All date args must be in \"YYYY-MM-DD\" format.")


def _parse_date_str(date_str):
    """
    >>> _parse_date_str("2011-01-02")
    datetime.date(2011, 1, 2)
    >>> _parse_date_str("Monday Jan 4th, 2011")
    Traceback (most recent call last):
    ...
    ValueError: time data 'Monday Jan 4th, 2011' does not match format '%Y-%m-%d'
    """
    return datetime.datetime.strptime(date_str, "%Y-%m-%d").date()


def get_date_from_string(date_str):
    """Returns a date object given a date_str.

    >>> get_date_from_string("2014-02-02")
    datetime.date(2014, 2, 2)
    >>> get_date_from_string("") == datetime.date.today()
    True

    :param date_str:
    :return:
    """
    if date_str:
        return _parse_date_str(date_str)
    else:
        return datetime.date.today()


def main():
    options, args = benchline.args.go(__doc__, usage="usage: %%prog [options] YYYY-MM-DD [YYYY-MM-DD]\n%s" % __doc__,
                               validate_args=validate_args)
    date1 = get_date_from_string(benchline.args.get_arg(args, 0))
    date2 = get_date_from_string(benchline.args.get_arg(args, 1))
    six.print_(ago.human(date1 - date2))


if __name__ == "__main__":
    main()



# !/usr/bin/env python

from setuptools import setup, find_packages

# we only use the subset of markdown that is also valid reStructuredText so
# that our README.md works on both github (markdown) and pypi (reStructuredText)
with open("README.md") as rm_file:
    long_description = rm_file.read()

setup(name='benchline',
      version='0.1.1',
      description="Modules for important things I don't want to reinvent over and over again.",
      long_description=long_description,
      author='Paul D. Eden',
      author_email='paul@benchline.org',
      url='https://github.com/pauldeden/benchline',
      packages=find_packages(),
      data_files=[('', ['README.md', 'LICENSE']),
                  ('benchline/new_python_file', ['benchline/new_python_file/new_python_file.tmpl',
                                                 'benchline/new_python_file/new_class_file.tmpl',
                                                 'benchline/new_python_file/new_cmd_file.tmpl'])],
      test_suite="benchline.test",
      license="MIT",
      install_requires=['requests',
                        'simplejson',
                        'decorator',
                        'ago', 'six'],
      zip_safe=False,
      entry_points={
          'console_scripts': [
              'date_diff = benchline.date_diff:main',
              'python_deploy = benchline.python_deploy:main',
              'python_install = benchline.python_install:main',
              'hours_seconds_2_hours = benchline.hours_seconds_2_hours:main',
              'countdown_generator = benchline.countdown_generator:main',
              'sum_timelog = benchline.sum_timelog:main',
              'new_setup_py = benchline.new_setup_py:main',
              'new_python_file = benchline.new_python_file.new_python_file:main']}
)



# !/usr/bin/python
#
# Author: Paul D. Eden <paul@benchline.org>
# Created: 2014-03-20
"""
Command-line argument processing functions
"""
from optparse import OptionParser


def make_parser(doc="", usage=None, add_options=True):
    """Make a parser object.

    :param doc:
    :param usage:
    :param add_options: default True whether to add -v and -t options by default
    :return: None
    """
    if not usage:
        usage = "usage: %%prog [options]\n%s" % doc
    parser = OptionParser()
    if usage:
        parser.set_usage(usage)
    if add_options:
        parser.add_option("-v", "--verbose", action="store_true", help="run gregariously")
        parser.add_option("-t", "--doctest", action="store_true", help="run doctests and exit")
    return parser


def triage(parser, main=None, validate_args=None):
    """Triage the command-line presented.

    :param parser:
    :param main:
    :param validate_args:
    :return:
    """
    options, args = parser.parse_args()

    if validate_args:
        validate_args(parser, options, args)
    if options.doctest or not main:  # run the tests if there is no main callback
        import doctest

        doctest.testmod(verbose=options.verbose)
    else:
        if main:
            main()
    return options, args


def go(doc="", usage=None, validate_args=None):
    """Wrapper function to do everything

    :param doc:
    :param usage:
    :param validate_args:
    :return: options, args
    """
    parser = make_parser(doc, usage)
    return triage(parser, validate_args=validate_args)


def get_arg(args, num):
    """Returns the num value from args
    without throwing an annoying exception
    when num is too large.

    >>> get_arg([0], 0)
    0
    >>> get_arg([0], 1)
    ''
    >>> get_arg([0, 1, 2, 3], 5)
    ''
    >>> get_arg([0, 1, 2, 3], 2)
    2

    :param num:
    :return: string arg value
    """
    if len(args) >= num + 1:
        return args[num]
    else:
        return ""


if __name__ == "__main__":
    go(__doc__)
